/**
 * @module dixim
 * @submodule download
 */
dixim.namespace('dixim.download');

/**
 * @module download
 * @submodule download-wizard-dialog-controller
 */

(function() {
var DownloadWizardDialogController;
var prototype;
var superConstructor;
var DownloadWizardSelectDstDialogView = dixim.download.DownloadWizardSelectDstDialogView;
var DownloadWizardTimingDialogView = dixim.download.DownloadWizardTimingDialogView;
var DownloadWizardConfirmDialogView = dixim.download.DownloadWizardConfirmDialogView;
var DownloadWizardStatusDialogView = dixim.download.DownloadWizardStatusDialogView;
var encodeId = dixim.xml.encodeId;
var EventTarget = dixim.event.EventTarget;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var ApiClientAutoDownload = dixim.common.ApiClientAutoDownload;
var PinManager = dixim.ctt.util.PinManager;

/**
 * @class DownloadWizardDialogController
 * @constructor
 * @param {Object} configs
 */
DownloadWizardDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(DownloadWizardDialogController, EventTarget);
dixim.download.DownloadWizardDialogController = DownloadWizardDialogController;
prototype = DownloadWizardDialogController.prototype;
superConstructor = DownloadWizardDialogController.superConstructor;

var NEXT = 'next';
var PREV = 'prev';
var START = 'start';
var CANCEL = 'cancel';
var CLOSE = 'close';
var FINISH = 'finish';
var SELECT_DIRECTORY = 'selectDirectory';

var BROWSING_STATUS = {
    STOPPED: 0,
    STOPPING: 1,
    BROWSING: 2,
};
var ROOT_OBJECTID = 'FS-Folder';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientTransfer();
    this._model.addEventListener('browse', this._successBrowse, this);
    this._model.addEventListener('browseError', this._errorBrowse, this);
    this._model.addEventListener('download', this._successDownload, this);
    this._model.addEventListener('downloadError', this._errorDownload, this);
    this._model.addEventListener('cancelTransfer', this._successCancelTransfer, this);
    this._model.addEventListener('cancelTransferError', this._errorCancelTransfer, this);

    this._model2 = new ApiClientAutoDownload();
    this._model2.addEventListener('fetchDstStorageList', this._successFetchDstStorageList, this);
    this._model2.addEventListener('fetchDstStorageListError', this._errorFetchDstStorageList, this);

    this._selectDstView;
    this._timingView;
    this._confirmView;
    this._statusView;
    this._contentHeight = ['300', 'px'].join('');
    this._statusTimer = null;
    this._transferStatus = {};
    this._unknownContents = [];
    this._browseRequestData;
    this._browsingStack = [];
    this._browsingStatus = BROWSING_STATUS.STOPPED;
    this._storagePaths = [];

    this._downloadConfigs = {};
    this._downloadConfigs.dataList = configs.dataList;
    this._downloadConfigs.serverInfo = configs.serverInfo;
};

/**
 * @method start
 */
prototype.start = function() {
    this._model.addEventListener('fetchTransferStatus', this._successCheckFetchTransferStatus, this);
    this._model.addEventListener('fetchTransferStatusError', this._errorCheckFetchTransferStatus, this);

    this._fetchTransferStatus();
};

/**
 * @method stop
 */
prototype.stop = function() {
    if (typeof this._statusView !== 'undefined') this._statusView.hide();
    if (typeof this._confirmView !== 'undefined') this._confirmView.hide();
    if (typeof this._timingView !== 'undefined') this._timingView.hide();
    if (typeof this._selectDstView !== 'undefined') this._selectDstView.hide();
};

prototype._successCheckFetchTransferStatus = function(e) {
    var query = {};
    var results = e.args.results;
    var resultsLen = results.length;
    var isTransferring = false;

    this._model.removeEventListener('fetchTransferStatus', this._successCheckFetchTransferStatus, this);
    this._model.removeEventListener('fetchTransferStatusError', this._errorCheckFetchTransferStatus, this);

    for (var i = 0; i < resultsLen; i += 1) {
        if (results[i].status === 'transferring') {
            isTransferring = true;
            break
        }
    }
    if (isTransferring) {
        this._downloadConfigs.dataList = results;
        this._viewStatus(results);
    } else {
        this._viewSelectDst();
    }
};

prototype._errorCheckFetchTransferStatus = function() {
    this._model.removeEventListener('fetchTransferStatus', this._successCheckFetchTransferStatus, this);
    this._model.removeEventListener('fetchTransferStatusError', this._errorCheckFetchTransferStatus, this);
};

prototype._viewSelectDst = function() {
    var query = {};

    query.contentHeight = this._contentHeight;

    this._selectDstView = new DownloadWizardSelectDstDialogView(query);
    this._selectDstView.addEventListener(SELECT_DIRECTORY, this._clickDirectory, this);
    this._selectDstView.addEventListener(PREV, this._clickPrev, this);
    this._selectDstView.addEventListener(NEXT, this._clickNextSelectDst, this);
    this._selectDstView.addEventListener(CLOSE, this._clickClose, this);
    this._selectDstView.show();
    this._selectDstView.changeCheckbox();
    this._browsingStack.push(ROOT_OBJECTID);
    this._model2.fetchDstStorageList();
};

prototype._successFetchDstStorageList = function(e) {
    var storages = e.args.storages;

    this._storagePaths = [];
    for (var i = 0, len = storages.length; i < len; i += 1) {
        this._storagePaths.push(storages[i].containerId);
    }
    this._browseAfterStop(ROOT_OBJECTID);
};

prototype._errorFetchDstStorageList = function() {
    this._errorBrowse();
};

prototype._viewBrowseDstContents = function(objectId) {
    this._selectDstView.cleanList();
    if (objectId === ROOT_OBJECTID) {
        this._selectDstView.setReturnButton();
    } else {
        this._selectDstView.setReturnButton(PREV);
    }
    this._selectDstView.changeCheckbox();
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._browseRequest(objectId, 0);
};

prototype._browseRequest = function(objectId, startingIndex) {
    var query = {};
    var pin = PinManager.get();

    query.id = objectId || ROOT_OBJECTID;
    query.starting_index = startingIndex;
    query.filter = 'container';
    if (typeof pin !== 'undefined') query.pin_code = pin;

    this._browseRequestData = query;
    this._model.browse(query);
};

prototype._successBrowse = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;
    var totalReturned = e.args.numberReturned;
    var totalMatches = e.args.totalMatches;
    var startingIndex = this._browseRequestData.starting_index;
    var isStopping = (this._browsingStatus === BROWSING_STATUS.STOPPING);
    var isEndBrowsing = ((totalReturned + startingIndex) === totalMatches);
    var isMatchId = true;

    if (resultsLen !== 0) isMatchId = (this._browseRequestData.id === results[0].parentId);

    if (totalMatches === 0) {
        this._selectDstView.noRow();
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }
    if (isStopping || (!isMatchId)) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }

    for (var i = 0; i < resultsLen; i += 1) {
        if (this._browseRequestData.id === ROOT_OBJECTID) {
            if (this._storagePaths.indexOf(results[i].id) === -1) {
                continue;
            }
        }
        this._selectDstView.addRow(results[i]);
    }

    if (isEndBrowsing) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }

    totalReturned += this._browseRequestData.starting_index;
    this._browseRequest(this._browseRequestData.id, totalReturned);
};

prototype._browseAfterStop = function(id) {
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._browsingStatus = BROWSING_STATUS.STOPPING;
    }

    if (this._browsingStatus !== BROWSING_STATUS.STOPPED) {
        setTimeout(dixim.proxy(function() {
            this._browseAfterStop(id)
        }, this), 100);
    } else {
        this._selectDstView.cleanList();
        this._viewBrowseDstContents(id);
    }
};

prototype._errorBrowse = function() {
    this._selectDstView.error(_('download_wizard_select_dst_browse_error_description'));
    this._clickClose();
    this._browsingStatus = BROWSING_STATUS.STOPPED;
};

prototype._clickDirectory = function(e) {
    this._browsingStack.push(e.args.id);
    this._browseAfterStop(e.args.id);
};

prototype._clickPrev = function() {
    var objectId;

    this._browsingStack.pop();
    objectId = this._browsingStack.pop();

    if (typeof objectId === 'undefined') {
        this._browsingStack.push(ROOT_OBJECTID);
        this._browseAfterStop(ROOT_OBJECTID);
    } else {
        this._browsingStack.push(objectId);
        this._browseAfterStop(objectId);
    }
};

prototype._clickNextSelectDst = function(e) {
    this._downloadConfigs.dst = e.args;
    this._viewTiming();
};

prototype._viewTiming = function() {
    var query = {};

    query.contentHeight = this._contentHeight;
    query.doubleDialog = true;

    this._timingView = new DownloadWizardTimingDialogView(query);
    this._timingView.addEventListener(NEXT, this._clickNextTiming, this);
    this._timingView.show();
};

prototype._clickNextTiming = function(e) {
    var startTime = e.args.startTime;

    if (typeof startTime !== 'undefined') {
        this._downloadConfigs.startTime = e.args.startTime;
    } else {
        delete this._downloadConfigs.startTime;
    }
    this._viewConfirm();
};

prototype._viewConfirm = function() {
    var query = {};

    query.contentHeight = this._contentHeight;
    query.downloadConfigs = this._downloadConfigs;
    query.doubleDialog = true;

    this._confirmView = new DownloadWizardConfirmDialogView(query);
    this._confirmView.addEventListener(START, this._clickStartConfirm, this);
    this._confirmView.show();
};

prototype._clickStartConfirm = function(e) {
    this._downloadConfigs.isFirstSd = e.args.isFirstSd;
    this._viewStatus();
};

prototype._viewStatus = function(results) {
    var query = {};

    query.contentHeight = this._contentHeight;
    query.doubleDialog = true;
    if (typeof results !== 'undefined') {
        query.doubleDialog = false;
        query.srcType = results[0].src_type;
        query.dstType = results[0].dst_type;
    }

    this._model.addEventListener('fetchTransferStatus', this._successFetchTransferStatus, this);
    this._model.addEventListener('fetchTransferStatusError', this._errorFetchTransferStatus, this);

    this._statusView = new DownloadWizardStatusDialogView(query);
    this._statusView.addEventListener(CANCEL, this._clickCancelStatus, this);
    this._statusView.addEventListener(CLOSE, this._clickClose, this);
    this._statusView.show();

    if (typeof results !== 'undefined') {
        this._successDownload();
    } else {
        this._download();
    }
};

prototype._download = function() {
    var query = {};
    var dataList = this._downloadConfigs.dataList;
    var dataListLen = dataList.length;
    var startTime = this._downloadConfigs.startTime;
    var udn = this._downloadConfigs.serverInfo.udn;
    var isFirstSd = this._downloadConfigs.isFirstSd;

    query.src_udn = udn;
    query.src_ids = [];
    query.dst_id = this._downloadConfigs.dst.id;
    for (var i = 0; i < dataListLen; i += 1) {
        query.src_ids.push(dataList[i].id);
    }
    if (typeof startTime !== 'undefined') query.start_time = startTime;
    query.res_criteria = (isFirstSd) ? 'lowest' : 'original';
    this._model.download(query);
};

prototype._successDownload = function() {
    var startTime = this._downloadConfigs.startTime;
    var dataList = this._downloadConfigs.dataList;
    var dataListLen = dataList.length;

    this._statusView.stopIndicator();
    if (typeof startTime !== 'undefined') {
        this._statusView.setStatusDescription(_('download_wizard_status_finish_reservation_transfer'), false);
        this._statusView.addDownloadReservationContents();
    } else {
        this._statusView.setStatusDescription(_('download_wizard_status_transferring'), true);
        this.lastResults = [];
        for (var i = 0; i < dataListLen; i += 1) {
            this._statusView.addDownloadContent(encodeId(dataList[i].id), dataList[i].title);
        }
        this._statusTimer = setInterval(dixim.proxy(this._fetchTransferStatus, this), 1000);
    }
};

prototype._errorDownload = function(e) {
    var error_code = e.args.g;

    this._statusView.stopIndicator();
    switch(error_code) {
        case 470:
            this._statusView.addDownloadReservationDuplicateErrorContents();
            break;
        default:
            this._statusView.addDownloadErrorContents();
            break;
    }
};

prototype._isUnknownValue = function(result) {
    if (typeof result.transferred !== 'number') return true;
    if (typeof result.total !== 'number') return true;
    if (result.total < 1) return true;
};

prototype._statusManager = function() {
    var STATUS = {
        UNKNOWN: 0x0000,
        ERROR: 0x0001,
        NOSPACE: 0x0002,
        UNSUPPORT: 0x0004,
        CANCEL: 0x0008,
        DONE: 0x0010,
    };
    var s = STATUS.UNKNOWN;;

    return {
        set: function(name) {
            s = s | STATUS[name];
        },
        unset: function(name) {
            s = (s & (~STATUS[name]));
        },
        isError: function() {
            if ((s & STATUS.ERROR) !== 0) return true;
            if ((s & STATUS.NOSPACE) !== 0) return true;
            if ((s & STATUS.UNSUPPORT) !== 0) return true;
            return false;
        },
        isMatch: function(name) {
            return ((s & STATUS[name]) !== 0);
        },
    };
};

prototype._successFetchTransferStatus = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;
    var percentage = 0;
    var isError = false;
    var errorCount = 0;
    var statusManager = new this._statusManager();

    statusManager.set('DONE');

    for (var i = 0; i < resultsLen; i += 1) {
        isError = false;
        if (typeof this._unknownContents[i] === 'undefined') {
            this._unknownContents[i] = {};
        }
        if (results[i].status === 'error') {
            statusManager.set('ERROR');
            isError = true;
            this._transferStatus.isError = true;
        }
        if (results[i].status === 'errorNoSpace') {
            statusManager.set('NOSPACE');
            isError = true;
            this._transferStatus.isError = true;
        }
        if (results[i].status === 'errorUnsupported') {
            statusManager.set('UNSUPPORT');
            isError = true;
            this._transferStatus.isError = true;
        }
        if (isError) {
            this._statusView.errorProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''));
            errorCount += 1;
            continue;
        }

        if (results[i].status !== 'done') statusManager.unset('DONE');
        if (results[i].status === 'cancelled') statusManager.set('CANCEL');

        if (this.timer === null) return;

        if (results[i].transferred && (! this._isUnknownValue(results[i]))) {
            if (results[i].transferred > results[i].total) {
                this._statusView.setRemainingTime(['#progress-bar-', encodeId(results[i].id)].join(''), 0);
                percentage = 100;
            } else {
                percentage = results[i].transferred / results[i].total * 100;
                if (typeof this.lastResults[i] === 'undefined' || typeof this.lastResults[i].transferTime === 'undefined') {
                    results[i].transferTime = 1;
                } else {
                    results[i].transferTime = this.lastResults[i].transferTime + 1;
                }
                results[i].transferSpeed = results[i].transferred / results[i].transferTime;
                this._statusView.setRemainingTime(['#progress-bar-', encodeId(results[i].id)].join(''), (results[i].total - results[i].transferred) / results[i].transferSpeed);
            }
            this._statusView.setProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''), percentage);
        }

        if (typeof results[i-1] !== 'undefined' && results[i-1].status !== 'done') {
            this._statusView.setProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''), 0);
            this._statusView.unknownProgressBar(['#progress-bar-', encodeId(results[i].id), ' #progress-description'].join(''));
            continue;
        }
        if (this._isUnknownValue(results[i])) {
            if (typeof this._unknownContents[i].transferred === 'undefined') {
                this._unknownContents[i].transferred = 1;
                this._unknownContents[i].total = 10;
            }
            this._unknownContents[i].transferred += 1;
            percentage = this._unknownContents[i].transferred / this._unknownContents[i].total * 100;
            this._statusView.setProgressBar(['#progress-bar-', encodeId(results[i].id)].join(''), percentage);
            this._statusView.unknownProgressBar(['#progress-bar-', encodeId(results[i].id), ' #progress-description'].join(''));
        }
    }
    this.lastResults = results;

    if(statusManager.isMatch('DONE')) {
        this._finishDownload();
    }
    if (statusManager.isMatch('CANCEL')) {
        this._successCancelTransfer();
    }
    if (errorCount !== resultsLen) return;

    if (statusManager.isMatch('ERROR')) {
        this._statusView.setStatusDescription(_('download_wizard_status_error'), false, true);
        return;
    }
    if (statusManager.isMatch('NOSPACE')) {
        this._statusView.setStatusDescription(_('download_wizard_status_nospace_error'), false, true);
        return;
    }
    if (statusManager.isMatch('UNSUPPORT')) {
        this._statusView.setStatusDescription(_('download_wizard_status_unsupported_error'), false, true);
        return;
    }
};

prototype._errorFetchTransferStatus = function(e) {
    if (typeof this._transferStatus.reached_count === 'undefined') this._transferStatus._error_reached_count = 0;
    this._transferStatus._error_reached_count += 1;

    if (this._transferStatus._error_reached_count === 10) {
        this._stopTransferStatus();
        this._statusView.setStatusDescription(_('download_wizard_status_error'), false, true);
    }
};

prototype._finishDownload = function() {
    if (this._transferStatus._error_flag) {
        this._statusView.setStatusDescription(_('download_wizard_status_maybe_error'), true);
    } else {
        this._statusView.setStatusDescription(_('download_wizard_status_finish_transfer'), false);
    }
    this._statusView.finishedDownload();
    this._stopTransferStatus();
    this.dispatchEvent(new Event(FINISH));
};

prototype._fetchTransferStatus = function() {
    this._model.fetchTransferStatus();
};

prototype._stopTransferStatus = function() {
    clearInterval(this._statusTimer);
    this._statusTimer = null;
};

prototype._clickCancelStatus = function() {
    this._statusView.startIndicator();
    this._model.cancelTransfer();
};

prototype._successCancelTransfer = function() {
    var resultsLen = this.lastResults.length;

    this._statusView.setStatusDescription(_('download_wizard_status_cancel'));
    this._statusView.stopIndicator();
    this._statusView.finishedDownload();
    this._stopTransferStatus();

    for (var i = 0; i < resultsLen; i += 1) {
        if (this.lastResults[i].status !== 'done') {
            this._statusView.errorProgressBar(['#progress-bar-', encodeId(this.lastResults[i].id)].join(''));
        }
    }
};

prototype._errorCancelTransfer = function() {
    this._statusView.setStatusDescription(_('download_wizard_status_error'), false, true);
};

prototype._clickClose = function() {
    this.stop();
    this.dispatchEvent(new Event(CLOSE));
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('download_wizard_title');
    query.description = text;
    query.doubleDialog = true;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
