/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule language-controller
 */

(function() {
var LanguageController;
var prototype;
var superConstructor;
var LanguageView = dixim.server.LanguageView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var BlockUi = dixim.common.BlockUi;
var ApiClientServer = dixim.common.ApiClientServer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var ConfirmModalDialogView = dixim.common.ConfirmModalDialogView;

/**
 * @class LanguageController
 * @constructor
 * @param {Object} configs
 */
LanguageController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(LanguageController, EventTarget);
dixim.server.LanguageController = LanguageController;
prototype = LanguageController.prototype;
superConstructor = LanguageController.superConstructor;

var APPLY = 'apply';
var PREV = 'prev';
var OK = 'ok';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'language'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('fetchLanguage', this._successFetchLanguage, this);
    this._model.addEventListener('fetchLanguageError', this._errorFetchLanguage, this);
    this._model.addEventListener('setLanguage', this._successSetLanguage, this);
    this._model.addEventListener('setLanguageError', this._errorSetLanguage, this);

    this._view = new LanguageView(configs);
    this._view.addEventListener(APPLY, this._clickApply, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._parentContext = configs.parentContext;
    this._blockUi = new BlockUi();
    this._languageInfo = {};
    this._dialog;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();

    this._blockUi.block();
    this._model.fetchLanguage();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successFetchLanguage = function(e) {
    var language = e.args.language;

    this._blockUi.unblock();
    this._view.setLanguage(language);
};

prototype._errorFetchLanguage = function(e) {
    this._blockUi.unblock();
    this._errorDialog(_('server_language_fetch_error'));
    this._view.disableSetting();
};

prototype._setLanguage = function(language) {
    var query = {};

    this._blockUi.block();
    query.language = language;
    this._model.setLanguage(query);
};

prototype._successSetLanguage = function() {
    this._blockUi.unblock();
    this._restartDms();
};

prototype._errorSetLanguage = function() {
    this._blockUi.unblock();
    this._errorDialog(_('server_language_set_error'));
};

prototype._restartDms = function() {
    var query = {};
    var errorStopStart = $.proxy(function() {
        this._blockUi.unblock();
        this._errorDialog(_('server_language_set_error'));
    }, this);
    var successStop = $.proxy(function() {
        var query = {};
        var successStart = $.proxy(function() {
            this._blockUi.unblock();
        }, this);

        query.status = 'start';
        query.success = successStart;
        query.error = errorStopStart;
        this._model.changeDmsStatus(query);
    }, this);

    this._blockUi.block();
    query.status = 'stop';
    query.success = successStop;
    query.error = errorStopStart;
    this._model.changeDmsStatus(query);
};

prototype._clickApply = function(e) {
    var query = {};

    this._languageInfo.language = e.args.language;
    query.id = this._id;
    query.title = _('server_menu_language');
    query.description = _('server_language_set_check');
    this._dialog = new ConfirmModalDialogView(query);
    this._dialog.addEventListener(OK, this._clickOk, this);
    this._dialog.show();
};

prototype._clickOk = function() {
    this._dialog.hide();
    this._setLanguage(this._languageInfo.language);
    this._languageInfo = {};
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('server_menu_language');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
