/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule pairing-controller
 */

(function() {
var PairingController;
var prototype;
var superConstructor;
var PairingView = dixim.server.PairingView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientServer = dixim.common.ApiClientServer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var DeletionModalDialogView = dixim.common.DeletionModalDialogView;
var BlockUi = dixim.common.BlockUi;

/**
 * @class PairingController
 * @constructor
 * @param {Object} configs
 */
PairingController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(PairingController, EventTarget);
dixim.server.PairingController = PairingController;
prototype = PairingController.prototype;
superConstructor = PairingController.superConstructor;

var SELECT = 'select';
var DELETE = 'delete';
var PREV = 'prev';
var YES = 'yes';
var COMPLETE = 'complete';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'pairing'].join('');

    this._model = new ApiClientServer();
    this._model.addEventListener('fetchPairingDevice', this._successFetchPairingDevice, this);
    this._model.addEventListener('fetchPairingDeviceError', this._errorFetchPairingDevice, this);
    this._model.addEventListener('removePairingDevice', this._successRemovePairingDevice, this);
    this._model.addEventListener('removePairingDeviceError', this._errorRemovePairingDevice, this);

    this._view = new PairingView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(DELETE, this._clickDelete, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._processInfo = {};
    this._title = _('server_menu_pairing');
    this._parentContext = configs.parentContext;
    this._selectedRegisteredDevices = [];
    this._currentRegisteredDevices = [];
    this._dialog;
    this._blockUi = new BlockUi();
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._fetchPairingDevice();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchPairingDevice = function() {
    this._blockUi.block();
    this._model.fetchPairingDevice();
};

prototype._successFetchPairingDevice = function(e) {
    var registeredDevices = e.args.registeredDevices;
    var registeredDevicesLen = registeredDevices.length;

    this._currentRegisteredDevices = [];
    this._blockUi.unblock();
    if (registeredDevicesLen === 0) {
        this._view.error(_('server_pairing_nothing'));
        return;
    }
    for (var i = 0; i < registeredDevicesLen; i += 1) {
        this._currentRegisteredDevices[i] = registeredDevices[i];
        this._view.addRow(registeredDevices[i]);
    }
};

prototype._errorFetchPairingDevice = function() {
    this._blockUi.unblock();
    this._view.error(_('server_pairing_error_fetch'));
};

prototype._successRemovePairingDevice = function(e) {
    this._dialog.decrementDescription();
};

prototype._errorRemovePairingDevice = function(e) {
    this._processInfo.error = true;
    this._dialog.decrementDescription();
};

prototype._deleteRegisteredDevices = function() {
    var selectedRegisteredDevicesLen = this._selectedRegisteredDevices.length;
    var query = {};

    this._processInfo.error = false;
    for (var i = 0; i < selectedRegisteredDevicesLen; i += 1) {
        query.deviceId = this._selectedRegisteredDevices[i].deviceId;
        query.dtlaIdHash = this._selectedRegisteredDevices[i].dtlaIdHash;
        this._model.removePairingDevice(query);
    }
};

prototype._deletionDialog = function(data) {
    var query = {};

    query.id = this._id;
    query.title = this._title;
    query.description = _('server_pairing_deletion_description', data.length);
    query.contents = data;
    query.name_key = 'friendlyName';
    query.button_text = _('dialog_delete');
    this._dialog = new DeletionModalDialogView(query);
    this._dialog.addEventListener(YES, this._clickYes, this);
    this._dialog.addEventListener(COMPLETE, this._completeDialog, this);
    this._dialog.show();
};

prototype._completeDialog = function() {
    this._updateRegisteredDeviceList();
    if (this._processInfo.error) {
        this._errorDialog(_('server_pairing_error_remove'));
    }
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = this._title;
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

prototype._clickYes = function() {
    this._deleteRegisteredDevices();
};

prototype._clickSelect = function() {
    this._view.switchSelect();
};

prototype._clickDelete = function(e) {
    var idList = e.args.idList;
    var idListLen = idList.length;

    this._selectedRegisteredDevices = [];
    if (idListLen === 0) {
        this._errorDialog(_('server_pairing_no_select_delete'));
        return;
    }
    for (var i in idList) {
        this._selectedRegisteredDevices.push(this._currentRegisteredDevices[i]);
    }
    this._deletionDialog(this._selectedRegisteredDevices);
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._updateRegisteredDeviceList = function() {
    this._view.cleanRegisteredDeviceList();
    this._fetchPairingDevice();
};

}());
