/**
 * @module ctt
 * @submodule transaction-wizard-page-controller
 */
(function() {
var Event = dixim.event.Event;
var WizardPageController = dixim.ctt.WizardPageController;
var TransactionWizardPageView = dixim.ctt.TransactionWizardPageView;

var TransactionWizardPageController;
var prototype;
var superConstructor;

/**
 * @class TransactionWizardPageController
 * @extends WizardPageController
 * @constructor
 * @param {Object} configs
 */
TransactionWizardPageController = function(configs) {
  WizardPageController.call(this, configs);
};
dixim.inherit(TransactionWizardPageController, WizardPageController);
dixim.ctt.TransactionWizardPageController = TransactionWizardPageController;
prototype = TransactionWizardPageController.prototype;
superConstructor = TransactionWizardPageController.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  superConstructor.prototype._initialize.call(this, configs);
  this._session = null;
};

prototype._createView = function() {
  this._isDownloadMoveIn = this._wizard.getRegistryEntry('isDownloadMoveIn');
  this._view = new TransactionWizardPageView({
    isDownloadMoveIn: this._isDownloadMoveIn
  });
};

prototype.start = function() {
  var v;

  if (!!!this._view) {
    this._createView();
  }

  v = this._view;
  v.addEventListener('render', this._didRenderView, this);
  v.addEventListener('cancel', this._didCancel, this);
  v.addEventListener('close', this._didClickClose, this);
  v.addEventListener('retry', this._didClickRetry, this);
  v.addEventListener('back', this._didClickBack, this);

  superConstructor.prototype.start.call(this);
};

prototype._didRenderView = function(e) {
  var w;
  var model;

  if (!!!(w = this._wizard)) {
    throw new Error('not pushed into wizard stack');
  }

  model = w.getRegistryEntry('model');
  if (this._isDownloadMoveIn) {
    model.addEventListener('dnTransfer', this._didSucceedTransfer, this);
    model.addEventListener('dnTransferError', this._didFailTransfer, this);
  } else {
    model.addEventListener('transfer', this._didSucceedTransfer, this);
    model.addEventListener('transferError', this._didFailTransfer, this);
  }
  this._startTransaction();
};

prototype._startTransaction = function() {
  this._showActivityIndicator();
  this._sendRequest(this._makeRequest());
};

prototype._showActivityIndicator = function() {
  var v = this._view;
  var w = this._wizard;

  if (!this._isDeviceAlive()) {
    v.showWolTransactionActivityIndicator();
  } else if (w.getRegistryEntry('scheduled')){
    v.showScheduledTransactionActivityIndicator();
  } else {
    v.showActivityIndicator();
  }
};

prototype._makeRequest = function() {
  var w = this._wizard;
  var request = {};
  var dms;
  var device;
  var destLocalContainer;
  var storage;
  var targetItems;
  var scheduledDate = null;
  var shutdown;
  var makeItemIdList;

  makeItemIdList = function(items) {
    var len = items.length;
    var i;
    var ret = [];

    for (i = 0; i < len; i += 1) {
      ret.push(items[i].id);
    }
    return ret;
  };

  targetItems = w.getRegistryEntry('targetItems');
  request.srcIds = makeItemIdList(targetItems);

  shutdown = w.getRegistryEntry('shutdownOnceFinished');
  request.shutdown = shutdown;

  scheduledDate = w.getRegistryEntry('scheduledDate');
  if (!!scheduledDate) {
    request.startTime = scheduledDate;
  }

  if (this._isDownloadMoveIn) {
    device = w.getRegistryEntry('sourceDevice');
    request.srcUdn = device.id;
    destLocalContainer = w.getRegistryEntry('destLocalContainer');
    request.destId = destLocalContainer.id;
  } else {
    dms = w.getRegistryEntry('dms');
    request.destUdn = dms.udn;
    storage = w.getRegistryEntry('destStorage');
    if (!!storage) {
      request.destId = storage.id;
    }
  }

  return request;
};

prototype._sendRequest = function(request) {
  var model;

  model = this._wizard.getRegistryEntry('model');
  if (!!this._session) {
    this._session.cancel();
  };
  if (this._isDownloadMoveIn) {
    this._session = model.dnTransfer(request);
  } else {
    this._session = model.transfer(request);
  }
};

prototype._didSucceedTransfer = function(e) {
  var args = e.args;
  var session = args.session;
  var v = this._view;
  var w = this._wizard;

  if (session !== this._session) {
    return;
  }
  this._session = null;

  if (session.isCancelled()) {
    return;
  }

  v.hideActivityIndicator();
  if (w.getRegistryEntry('scheduled')) {
    v.showReservationComplete();
  } else {
    this._done();
  }
};

prototype._didFailTransfer = function(e) {
  var args = e.args;
  var errorCode = args.errorCode;
  var session = args.session;
  var w = this._wizard;
  var v = this._view;

  if (session !== this._session) {
    return;
  }
  this._session = null;

  if (session.isCancelled()) {
    return;
  }

  v.hideActivityIndicator();

  if (w.getRegistryEntry('scheduled')) {
    v.showReservationError(errorCode);
  } else {
    if (!this._isDeviceAlive()) {
      v.showWolError(errorCode);
    } else {
      v.showError(errorCode);
    }
  }
};

prototype._isDeviceAlive = function() {
  var d = null;
  var w = this._wizard;

  if (this._isDownloadMoveIn) {
    return true;
  } else {
    d = w.getRegistryEntry('dms');
  }
  return (!!d && d.alive);
};

prototype._didClickClose = function(e) {
  this._done();
};

prototype._done = function() {
  this._tearDown();
  this.dispatchEvent(new Event('done'));
};

prototype._didClickRetry = function(e) {
  this._startTransaction();
};

prototype._didClickBack = function(e) {
  var vc;
  var w;

  if (!!(w = this._wizard)) {
    this._tearDown();
    vc = w.popViewController();
    vc.destroy();
  }
};

prototype._tearDown = function() {
  var model;
  var w = this._wizard;

  if (!!this._session) {
    this._session.cancel();
  }
  model = w.getRegistryEntry('model');
  if (this._isDownloadMoveIn) {
    model.removeEventListener('dnTransfer', this._didSucceedTransfer, this);
    model.removeEventListener('dnTransferError', this._didFailTransfer, this);
  } else {
    model.removeEventListener('transfer', this._didSucceedTransfer, this);
    model.removeEventListener('transferError', this._didFailTransfer, this);
  }
};

}());
