/**
 * @module ctt
 * @submodule transfer-progress-view
 */
(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var TickerView = dixim.ctt.TickerView;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;

var TransferProgressView;
var ScreenState = null;
var InProgressState = null;
var CancelingState = null;
var DoneState = null;
var DoneWithCancellationState = null;

var prototype;
var superConstructor;

/**
 * @class TransferProgressView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 * @param {Boolean} [configs.showCloseButton=true]
 */
TransferProgressView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(TransferProgressView, ModalDialogView);
dixim.ctt.TransferProgressView = TransferProgressView;
prototype = TransferProgressView.prototype;
superConstructor = TransferProgressView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-transfer-progress-view-wrapper" />';
var TEMPLATE_BODY_CONTENT = ['<div class="ctt-transfer-progress-view">',
                               '<div class="ctt-transfer-progress-view-message-wrapper">',
                                 '<div class="ctt-transfer-progress-view-description"></div>',
                                 '<div class="ctt-transfer-progress-view-error"></div>',
                               '</div>',
                               '<div class="ctt-transfer-progress-view-status-list"></div>',
                             '</div>'].join('');

/**
 * Triggered when the page has been cancelled.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the page has been closed.
 *
 * @event close
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Boolean} [configs.showCloseButton=true]
 */
prototype._initialize = function(configs) {
  var doneButtons = [];
  var enableClose;

  configs  = configs || {};

  configs.contentBox = TEMPLATE_CONTENT_BOX;
  configs.bodyContent = TEMPLATE_BODY_CONTENT;
  configs.title = _('ctt_transfer_progress_view_title');

  enableClose = typeof configs.showCloseButton === 'undefined' ? true : configs.showCloseButton;
  if (enableClose) {
    doneButtons.push({
      // TODO: align: 'center',
      text: _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    });
  }
  this._doneButtons = doneButtons;

  this._inProgressButtons = [
    {
      // TODO: align: 'center',
      text: _('ctt_dialog_button_label_cancel'),
      click: $.proxy(this._didClickCancel, this)
    }
  ];

  this._descriptionNode = null;
  this._errorNode = null;
  this._ticker = null;
  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didClickClose = function(e) {
  this._state.didClickClose(this);
};

prototype._didClickCancel = function(e) {
  this._state.didClickCancel(this);
};

prototype._didOpenDialog = function(e) {
  var v = this._view;
  var ticker;

  superConstructor.prototype._didOpenDialog.call(this, e);

  ticker = new TickerView({
    containerNode: $('.ctt-transfer-progress-view-error', v),
    type: 'error'
  });
  ticker.addEventListener('hide', this._didChangeTickerVisibility, this);
  ticker.addEventListener('show', this._didChangeTickerVisibility, this);

  this._ticker = ticker;
  this._descriptionNode = $('.ctt-transfer-progress-view-description', v);

  this._adjustListHeight();
  this.changeState(InProgressState);
};

prototype._adjustListHeight = function() {
  var v = this._view;
  var list;
  var desc;

  list = $('.ctt-transfer-progress-view-status-list', v);
  desc = $('.ctt-transfer-progress-view-message-wrapper', v);
  list.height(Math.floor(v.height() - desc.outerHeight()));
};

prototype._didChangeTickerVisibility = function(e) {
  this._adjustListHeight();
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._state.didClickClose(this);
};

prototype.getBodyContent = function() {
  return $('.ctt-transfer-progress-view-status-list', this._view);
};

prototype.enterInProgressState = function() {
  this._state.enterInProgress(this);
};

prototype.enterDoneState = function() {
  this._state.enterDone(this);
};

prototype.enterDoneWithErrorState = function() {
  this._state.enterDoneWithError(this);
};

prototype.enterDoneWithCancellationState = function() {
  this._state.enterDoneWithCancellation(this);
};

prototype.enterCancelingState = function() {
  this._state.enterCanceling(this);
};

prototype.updateText = function(desc, error) {
  this._updateText(desc, error);
};

prototype._updateText = function(desc, error) {
  this._descriptionNode.empty().text(desc || '');
  if (!!error) {
    this._ticker.setText(error);
    this._ticker.show();
  } else {
    this._ticker.hide(false);
  }
};

prototype.switchButtonsToInProgress = function() {
  this.setButtons(this._inProgressButtons);
};

prototype.switchButtonsToDone = function() {
  this.setButtons(this._doneButtons);
};

prototype.restartTransfer = function() {
    $('.ctt-transfer-progress-view-status-list ul').empty();
    this.changeState(InProgressState);
};

/**
 * @method changeState
 * @param {ScreenState} state
 */
prototype.changeState = function(state) {
  this._state = state;
  state.enter(this);
  this._didChangeState();
};

prototype._didChangeState = function() {
  this._adjustListHeight();
};

prototype.destroy = function() {
  if (!!this._ticker) {
    this._ticker.destroy();
  }
  superConstructor.prototype.destroy.call(this);
};

/**
 * Abstract class of the screen state.
 *
 * @class ScreenState
 * @uses TransferProgressView
 * @constructor
 * @static
 * @private
 */
ScreenState = {
  /**
   * @method enter
   * @param {TransferProgressView} context
   */
  enter: function(context) {},

  /**
   * @method enterInProgress
   * @param {TransferProgressView} context
   */
  enterInProgress: function(context) {},

  /**
   * @method enterDone
   * @param {TransferProgressView} context
   */
  enterDone: function(context) {},

  /**
   * @method enterDoneWithCancellation
   * @param {TransferProgressView} context
   */
  enterDoneWithCancellation: function(context) {},

  /**
   * @method enterDoneWithError
   * @param {TransferProgressView} context
   */
  enterDoneWithError: function(context) {},

  /**
   * @method enterCanceling
   * @param {TransferProgressView} context
   */
  enterCanceling: function(context) {},

  /**
   * @method didClickClose
   * @param {TransferProgressView} context
   */
  didClickClose: function(context) {},

  /**
   * @method didClickCancel
   * @param {TransferProgressView} context
   */
  didClickCancel: function(context) {}
};

/**
 * @class DoneState
 * @uses TransferProgressView
 * @constructor
 * @private
 * @extends ScreenState
 * @static
 */
DoneState = dixim.extend(dixim.create(ScreenState), {
  enter: function(context) {
    context.updateText(_('ctt_transfer_progress_view_description_done'), null);
    context.switchButtonsToDone();
  },
  didClickClose: function(context) {
    context.dispatchEvent(new Event(CLOSE));
  }
});

/**
 * @class InProgressState
 * @uses TransferProgressView
 * @constructor
 * @private
 * @extends ScreenState
 * @static
 */
InProgressState = dixim.extend(dixim.create(ScreenState), {
  enter: function(context) {
    context.updateText(_('ctt_transfer_progress_view_description_in_progress'), null);
    context.switchButtonsToInProgress();
  },
  enterDone: function(context) {
    context.changeState(DoneState);
  },
  enterCanceling: function(context) {
    context.changeState(CancelingState);
  },
  enterDoneWithCancellation: function(context) {
    context.changeState(DoneWithCancellationState);
  },
  enterDoneWithError: function(context) {
    context.changeState(DoneWithErrorState);
  },
  didClickCancel: function(context) {
    context.dispatchEvent(new Event(CANCEL));
  },
  didClickClose: function(context) {
    context.dispatchEvent(new Event(CANCEL));
  }
});

/**
 * @class CancelingState
 * @uses TransferProgressView
 * @constructor
 * @private
 * @extends ScreenState
 * @static
 */
CancelingState = dixim.extend(dixim.create(ScreenState), {
  enter: function(context) {
    context.updateText(_('ctt_transfer_progress_view_description_canceling'), null);
    context.switchButtonsToInProgress();
  },
  enterDone: function(context) {
    context.changeState(DoneState);
  },
  enterDoneWithCancellation: function(context) {
    context.changeState(DoneWithCancellationState);
  }
});

/**
 * @class DoneWithCancellationState
 * @uses TransferProgressView
 * @constructor
 * @private
 * @extends ScreenState
 * @static
 */
DoneWithCancellationState = dixim.extend(dixim.create(ScreenState), {
  enter: function(context) {
    context.updateText(_('ctt_transfer_progress_view_description_done_with_cancellation'), null);
    context.switchButtonsToDone();
  },
  didClickClose: function(context) {
    context.dispatchEvent(new Event(CLOSE));
  }
});

/**
 * @class DoneWithErrorState
 * @uses TransferProgressView
 * @constructor
 * @private
 * @extends ScreenState
 * @static
 */
DoneWithErrorState = dixim.extend(dixim.create(ScreenState), {
  enter: function(context) {
    context.updateText(_('ctt_transfer_progress_view_description_done_with_error'),
                     _('ctt_transfer_progress_view_error'));
    context.switchButtonsToDone();
  },
  didClickClose: function(context) {
    context.dispatchEvent(new Event(CLOSE));
  }
});

}());
