/**
 * @module ctt
 * @submodule ctt-api
 */
dixim.namespace('dixim.ctt.api');

/**
 * @module ctt-api
 * @submodule auto-delete-api-client-session
 */
(function() {
var AutoDeleteApiClientSession;
var prototype;

/**
 * Represents a session data of API action.
 *
 * @class AutoDeleteApiClientSession
 * @param {String} api API action name. (such as 'set' etc.)
 * @param {Object} query Query data used for the action.
 * @param {Object} [userData=null] Use-specific data.
 */
AutoDeleteApiClientSession = function(api, query, userData) {
  /**
   * @property api
   * @type {String} API name
   */
  this.api = api;

  /**
   * @property query
   * @type {Object} query data used for the request.
   */
  this.query = query;

  /**
   * @property userData
   * @type {Object} user-specific data;
   */
  this.userData = userData || null;

  /**
   * Request object.
   *
   * Some APIs don't set this value,
   * so please do null-check before accessing this property.
   *
   * @property request
   * @type {Object}
   */
  this.request = null;

  /**
   * @property connection
   * @type {jQuery}
   */
  this.connection = null;

  /**
   * @property _isCancelled
   * @private
   * @type {boolean}
   */
  this._isCancelled = false;
};
dixim.ctt.api.AutoDeleteApiClientSession = AutoDeleteApiClientSession;
prototype = AutoDeleteApiClientSession.prototype;

/**
 * Returns whether the session is cancelled or not.
 *
 * @method isCancelled
 * @return {boolean}
 */
prototype.isCancelled = function() {
  return this._isCancelled;
};

/**
 * Cancels session.
 *
 * Currently, it just make the session cancelled state.
 *
 * @method cancel
 */
prototype.cancel = function () {
  this._isCancelled = true;
};

}());

/**
 * @module ctt-api
 * @submodule contents-transfer-api-client
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Formatter = dixim.ctt.util.Formatter;
var AutoDeleteApiClientSession = dixim.ctt.api.AutoDeleteApiClientSession;
var Configs = dixim.ctt.Configs;
var $ = dixim.ctt.jQuery;

var AutoDeleteApiClient;
var prototype;

/**
 * Provides interfaces to Contents Transfer API.
 *
 * @class AutoDeleteApiClient
 * @param {Object} [configs]
 */
AutoDeleteApiClient = function(configs) {
  this._initialize(configs);
};
dixim.inherit(AutoDeleteApiClient, EventTarget);
dixim.ctt.api.AutoDeleteApiClient = AutoDeleteApiClient;
prototype = AutoDeleteApiClient.prototype;

/**
 * Triggered when the lock action was succeeded.
 *
 * @event lock
 */
var LOCK = 'lock';

/**
 * Triggered when the lock action was failed.
 *
 * @event lockError
 */
var LOCK_ERROR = 'lockError';

/**
 * Triggered when the unlock action was succeeded.
 *
 * @event unlock
 */
var UNLOCK = 'unlock';

/**
 * Triggered when the unlock action was failed.
 *
 * @event unlockError
 */
var UNLOCK_ERROR = 'unlockError';

/**
 * Triggered when the Auto Delete setting action was succeeded.
 *
 * @event autodelete
 */
var SET_STATUS = 'setAutoDeleteStatus';

/**
 * Triggered when the  Auto Delete setting action was failed.
 *
 * @event autodeleteError
 */
var SET_STATUS_ERROR = 'setAutoDeleteStatusError';

/**
 * Triggered when the Auto Delete setting action was succeeded.
 *
 * @event autodelete
 */
var GET_STATUS = 'getAutoDeleteStatus';

/**
 * Triggered when the  Auto Delete setting action was failed.
 *
 * @event autodeleteError
 */
var GET_STATUS_ERROR = 'getAutoDeleteStatusError';

/**
 * Triggered when the Auto Delete setting action was succeeded.
 *
 * @event autodelete
 */
var SET_LIMIT = 'setAutoDeleteLimit';

/**
 * Triggered when the  Auto Delete setting action was failed.
 *
 * @event autodeleteError
 */
var SET_LIMIT_ERROR = 'setAutoDeleteLimitError';

/**
 * Triggered when the Auto Delete setting action was succeeded.
 *
 * @event autodelete
 */
var GET_LIMIT = 'getAutoDeleteLimit';

/**
 * Triggered when the  Auto Delete setting action was failed.
 *
 * @event autodeleteError
 */
var GET_LIMIT_ERROR = 'getAutoDeleteLimitError';

/**
 * @method _initialize
 * @protected
 * @param {Object} [configs]
 * @param {String} [configs.pin]
 */
prototype._initialize = function(configs) {
  EventTarget.call(this);

  configs = configs || {};
  this._sessions = [];
  this._pin = typeof configs.pin === 'undefined' ? null : configs.pin;
};

prototype._makeResponseHandler = function(handler, args) {
  var slice = Array.prototype.slice;
  return $.proxy(function() {
    handler.apply(this, slice.call(arguments, 0).concat(args));
  }, this);
};

prototype._handleFailureResponse = function(session, errorCode, eventType) {
  dixim.array.remove(this._sessions, session);
  this.dispatchEvent(new Event(eventType, {session: session, errorCode: errorCode}));
};

/**
 * Invokes the lock action.
 *
 * @method lock
 * @param {Object} request
 * @param {String} request.id
 * @param {String} request.newName
 * @return {AutoDeleteApiClientSession}
 */
prototype.lock = function(request) {
  var api = 'items/removeProtect/set';
  var session;
  var req;
  var query = {};

  session = new AutoDeleteApiClientSession(api);

  query.objectIds = request.objectIds;
  query.protectFlag = request.protectFlag;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didLock, [session]),
    error: this._makeResponseHandler(this._didFailLock, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api_autodel'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didLock = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(LOCK, args);
  this.dispatchEvent(e);
};

prototype._didFailLock = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, LOCK_ERROR);
};

/**
 * Invokes the unlock action.
 *
 * @method unlock
 * @param {Object} request
 * @param {String} request.id
 * @param {String} request.newName
 * @return {AutoDeleteApiClientSession}
 */
prototype.unlock = function(request) {
  var api = 'items/removeProtect/set';
  var session;
  var req;
  var query = {};

  session = new AutoDeleteApiClientSession(api);

  query.objectIds = request.objectIds;
  query.protectFlag = request.protectFlag;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didUnlock, [session]),
    error: this._makeResponseHandler(this._didFailUnlock, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api_autodel'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didUnlock = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(UNLOCK, args);
  this.dispatchEvent(e);
};

prototype._didFailLock = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, UNLOCK_ERROR);
};

/**
 * Invokes the set auto delete status action.
 *
 * @method setstatus
 * @param {Object} request
 * @param {String} request.availability
 * @return {AutoDeleteApiClientSession}
 */
prototype.setstatus = function(request) {
  var api = 'settings/availability/set';
  var session;
  var req;
  var query = {};

  session = new AutoDeleteApiClientSession(api);

  query.availability = request.availability;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didSetAutoDeleteStatus, [session]),
    error: this._makeResponseHandler(this._didFailSetAutoDeleteStatus, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api_autodel'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didSetAutoDeleteStatus = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(SET_STATUS, args);
  this.dispatchEvent(e);
};

prototype._didFailSetAutoDeleteStatus = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, SET_STATUS_ERROR);
};

/**
 * Invokes the get auto delete status action.
 *
 * @method getstatus
 * @param {Object} request
 * @return {AutoDeleteApiClientSession}
 */
prototype.getstatus = function(request) {
  var api = 'settings/availability/get';
  var session;
  var req;

  session = new AutoDeleteApiClientSession(api);

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didGetAutoDeleteStatus, [session]),
    error: this._makeResponseHandler(this._didFailGetAutoDeleteStatus, [session]),
    dataType: 'json',
    context: this
  };

  session.request = request;
  session.connection = $.ajax([Configs.get('api_autodel'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didGetAutoDeleteStatus = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(GET_STATUS, args);
  this.dispatchEvent(e);
};

prototype._didFailGetAutoDeleteStatus = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, GET_STATUS_ERROR);
};

/**
 * Invokes the set auto delete limit action.
 *
 * @method setlimit
 * @param {Object} request
 * @param {String} request.freeSpaceThreshold
 * @return {AutoDeleteApiClientSession}
 */
prototype.setlimit = function(request) {
  var api = 'settings/freeSpaceThreshold/set';
  var session;
  var req;
  var query = {};

  session = new AutoDeleteApiClientSession(api);

  query.freeSpaceThreshold = request.freeSpaceThreshold;

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didSetAutoDeleteLimit, [session]),
    error: this._makeResponseHandler(this._didFailSetAutoDeleteLimit, [session]),
    dataType: 'json',
    context: this,
    data: query
  };

  session.request = request;
  session.query = query;
  session.connection = $.ajax([Configs.get('api_autodel'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didSetAutoDeleteLimit = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  e = new Event(SET_LIMIT, args);
  this.dispatchEvent(e);
};

prototype._didFailSetAutoDeleteLimit = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, SET_LIMIT_ERROR);
};


/**
 * Invokes the get auto delete limit action.
 *
 * @method getlimit
 * @param {Object} request
 * @return {AutoDeleteApiClientSession}
 */
prototype.getlimit = function(request) {
  var api = 'settings/freeSpaceThreshold/get';
  var session;
  var req;

  session = new AutoDeleteApiClientSession(api);

  req = {
    cache: false,
    type: 'POST',
    success: this._makeResponseHandler(this._didGetAutoDeleteLimit, [session]),
    error: this._makeResponseHandler(this._didFailGetAutoDeleteLimit, [session]),
    dataType: 'json',
    context: this
  };

  session.request = request;
  session.connection = $.ajax([Configs.get('api_autodel'), api].join('/'), req);
  this._sessions.push(session);

  return session;
};

prototype._didGetAutoDeleteLimit = function(data, textStatus, jqXhr, session) {
  var e;
  var args = {};

  dixim.array.remove(this._sessions, session);

  args.session = session;
  args.data = data;
  e = new Event(GET_LIMIT, args);
  this.dispatchEvent(e);
};

prototype._didFailGetAutoDeleteLimit = function(jqXhr, textStatus, errorThrown, session) {
  this._handleFailureResponse(session, jqXhr.status, GET_LIMIT_ERROR);
};

}());
