/**
 * @module ctt
 * @submodule contents-transfer-tool-controller
 */
(function(){
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var ContentsTransferToolView = dixim.ctt.ContentsTransferToolView;
var ApplicationToolbarView = dixim.ctt.ApplicationToolbarView;
var SearchBoxController = dixim.ctt.SearchBoxController;
var ContainerListingController = dixim.ctt.ContainerListingController;
var ContentListingController = dixim.ctt.ContentListingController;
var ContentListingControllerCriteria = dixim.ctt.ContentListingControllerCriteria;
var ErrorDialog = dixim.ctt.ErrorDialog;
var TransferWizardController = dixim.ctt.TransferWizardController;
var TransferStatusMonitor = dixim.ctt.TransferStatusMonitor;
var TransferProgressController = dixim.ctt.TransferProgressController;
var RenamingController = dixim.ctt.RenamingController;
var DeletionController = dixim.ctt.DeletionController;
var LockingController = dixim.ctt.LockingController;
var UnlockingController = dixim.ctt.UnlockingController;
var AutoDeleteController = dixim.ctt.AutoDeleteController;
var ParentalLockController = dixim.ctt.ParentalLockController;
var TransferHistoryController = dixim.ctt.TransferHistoryController;
var TransferScheduleWizardController = dixim.ctt.TransferScheduleWizardController;
var ConfirmationDialogController = dixim.ctt.ad.ConfirmationDialogController;
var CompressionSettingDialogController = dixim.ctt.CompressionSettingDialogController;
var _ = dixim.ctt.ResourceBundle.get;

var ContentsTransferToolController;
var prototype;

/**
 * Root controller of the Contents Transfer Tool.
 *
 * @class ContentsTransferToolController
 * @constructor
 * @param {Object} configs config object.
 * @param {String} configs.ContainerNode
 *  an ID of the tool container. (ex: '#container', '#wrapper')
 */
ContentsTransferToolController = function(configs) {
  this._initialize(configs);
};
dixim.ctt.ContentsTransferToolController = ContentsTransferToolController;
prototype = ContentsTransferToolController.prototype;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs config object.
 * @param {String} configs.containerNode
 */
prototype._initialize = function(configs) {
  this._pin = null;

  // TODO: view controller should have sticky flags or
  // something to become a topmost view controller permanently.
  this._modal = null;
  // TODO: ...end.

  this._containerNode = configs.containerNode;
  this._view = new ContentsTransferToolView({containerNode: configs.containerNode});
  this._transferStatusMonitor = new TransferStatusMonitor();

  this._appToolbarView = new ApplicationToolbarView({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_FOOTER_MENU,
    isAutoDelete: !!configs.isAutoDelete,
    isCompression: !!configs.isCompression
  });

  this._searchBox = new SearchBoxController({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_SEARCH_BOX
  });

  this._containerList = new ContainerListingController({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_CONTAINER_LIST,
    model: new ContentsTransferApiClient()
  });

  this._contentList = new ContentListingController({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_CONTENT_LIST,
    model: new ContentsTransferApiClient(),
    isAutoDelete: configs.isAutoDelete,
    isCompression: !!configs.isCompression
  });
};

/**
 * @method start
 */
prototype.start = function() {
  this._setEventListeners();
  this._view.show();
  this._appToolbarView.show();
  this._searchBox.start();
  this._containerList.start();
  this._contentList.start();
  this._transferStatusMonitor.start();
};

prototype._setEventListeners = function() {
  this._appToolbarView.addEventListener('render', this._didRenderApplicationToolbar, this);
  this._appToolbarView.addEventListener('compression_setting', this._didRequestConpressionSetting, this);
  this._appToolbarView.addEventListener('unlock', this._didRequestUnlock, this);
  this._appToolbarView.addEventListener('history', this._didRequestHistory, this);
  this._appToolbarView.addEventListener('schedule', this._didRequestSchedule, this);

  this._searchBox.addEventListener('search', this._didSearch, this);
  this._containerList.addEventListener('selectContainer', this._didSelectContainer, this);
  this._contentList.addEventListener('requestForTransferring', this._didRequestForTransferring, this);
  this._contentList.addEventListener('rename', this._didRequestRenaming, this);
  this._contentList.addEventListener('compression', this._didRequestCompression, this);
  this._contentList.addEventListener('delete', this._didRequestDeletion, this);
  this._contentList.addEventListener('lock', this._didRequestLocking, this);
  this._contentList.addEventListener('unlock', this._didRequestUnlocking, this);
  this._appToolbarView.addEventListener('autodelete', this._didRequestAutoDelSetting, this);

  this._transferStatusMonitor.addEventListener('transferring', this._didEnterTransferringState, this);
  this._transferStatusMonitor.addEventListener('canceling', this._didEnterTransferringState, this);
};

prototype._didRenderApplicationToolbar = function(e) {
  if (!!this._pin) {
    this._appToolbarView.disableUnlockButton();
  } else {
    this._appToolbarView.enableUnlockButton();
  }
};

prototype._didRequestConpressionSetting = function(e) {
  var csdc;

  csdc = new CompressionSettingDialogController({finish: dixim.proxy(this._reloadListControllers, this)});
  csdc.start();
};

prototype._didRequestUnlock = function(e) {
  var vc;

  if (!!this._model) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new ParentalLockController();
  vc.addEventListener('close', this._didCloseParentalLockDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseParentalLockDialog = function(e) {
  var pin = e.args.pin;

  if (!!pin) {
    this._appToolbarView.disableUnlockButton();
    this._setPin(pin);
  }
  this._modal.destroy();
  this._modal = null;
};

prototype._setPin = function(pin) {
  this._pin = pin;
  this._didChangePin();
};

prototype._didChangePin = function() {
  var pin = this._pin;

  this._contentList.setPin(pin);
  this._containerList.setPin(pin);
};

prototype._didRequestHistory = function() {
  var vc;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferHistoryController();
  vc.addEventListener('close', this._didCloseHistoryDialog, this);
  vc.setPin(this._pin);
  vc.start();
  this._modal = vc;
};

prototype._didCloseHistoryDialog = function(e) {
  this._modal.destroy();
  this._modal = null;
};

prototype._didRequestSchedule = function(e) {
  var vc;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferScheduleWizardController({
    model: new ContentsTransferApiClient({pin: this._pin})
  });
  // TODO: done should be close.
  vc.addEventListener('done', this._didCloseScheduleDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseScheduleDialog = function(e) {
  this._modal.destroy();
  this._modal = null;
};

prototype._didSearch = function(e) {
  var keyword = e.args.keyword;
  var criteria;

  if (!!!keyword) return;

  criteria = new ContentListingControllerCriteria();
  criteria.type = 'search';
  criteria.containerObject = {id: 'FS-Folder'};
  criteria.searchCriteria = {'title' : keyword};
  this._contentList.updateCurrentCriteria(criteria);
};

prototype._didSelectContainer = function(e) {
  var data = e.args.container;
  var criteria;

  criteria = new ContentListingControllerCriteria({containerObject: data, device: e.args.device});
  this._contentList.updateCurrentCriteria(criteria);
};

prototype._didRequestForTransferring = function(e) {
  var items = e.args.items;
  var error;
  var wizard;

  error = this._validateTransferTarget(items);
  if (!!error) {
    this._showErrorDialog(error);
    return;
  }

  // TODO: modal dialog priority
  if (!!this._modal) return;

  wizard = new TransferWizardController({
    targetItems: items,
    model: new ContentsTransferApiClient()
  });
  wizard.addEventListener('done', this._doneTransferWizard, this);
  wizard.addEventListener('cancel', this._didCancelTransferWizard, this);
  wizard.start();
  this._modal = wizard;
};

prototype._didEnterTransferringState = function(e) {
  this._startTransferProgress();
};

prototype._didCloseTransferProgress = function(e) {
  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  this._transferStatusMonitor.start();
  this._reloadListControllers();
};

prototype._doneTransferWizard = function(e) {
  var data = e.args.registry;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  // XXX: sometimes the transferring has done
  // before the transfer progress UI is opened.
  // Forcibly start the transfer progress UI to
  // show the transferring results.
  // TODO:
  if (!data.scheduled) {
    this._startTransferProgress();
    return;
  }
  this._reloadListControllers();
};

prototype._startTransferProgress = function() {
  var vc;

  this._transferStatusMonitor.stop();

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferProgressController({
    model: new ContentsTransferApiClient({pin: this._pin})
  });
  vc.addEventListener('close', this._didCloseTransferProgress, this);
  vc.start();

  this._modal = vc;
};

prototype._didCancelTransferWizard = function(e) {
  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  this._reloadListControllers();
};

prototype._validateTransferTarget = function(targets) {
  var len = targets.length;
  var i;
  var target;

  if (!len) {
    return 'ctt_error_dialog_transfer_no_media_selected';
  }

  for (i = 0; i < len; i += 1) {
    target = targets[i];
    if (target['class'] !== 'item' || !!!target['profile'] || !target['uploadable']) {
      return 'ctt_error_dialog_transfer_unsupported_media_selected';
    }
    if (target['restricted']) {
      return 'ctt_error_dialog_transfer_restricted_media_selected';
    }
  }

  return null;
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

prototype._didRequestRenaming = function(e) {
  var vc;
  var items;

  items = this._contentList.getSelectedItems();
  if (items.length === 0) {
    this._showErrorDialog('ctt_error_dialog_renaming_no_media_selected');
    return;
  }
  if (items.length > 1) {
    this._showErrorDialog('ctt_error_dialog_renaming_multi_titles_selected');
    return;
  }
  if (this._hasRestrictedItem(items)) {
    this._showErrorDialog('ctt_error_dialog_renaming_restricted_media_selected');
    return;
  }

  vc = new RenamingController({targetItem: items[0]});
  vc.addEventListener('close', this._didCloseRenamingDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseRenamingDialog = function(e) {
  if (e.args.renamed) {
    this._reloadListControllers();
  }
  this._modal.destroy();
  this._modal = null;
};

prototype._didRequestCompression = function(e) {
  var model = new ContentsTransferApiClient();

  model.addEventListener('fetchAvailabilityCompression', this._didCheckAvailability, this);
  model.fetchAvailabilityCompression();
};

prototype._didCheckAvailability = function(e) {
  var model;
  var availability = e.args.data.availability;

  if (availability) {
    model = new ContentsTransferApiClient();
    model.addEventListener('fetchPreset', this._didCreateCompressionDialog, this);
    model.fetchPreset();
  } else {
    this._showErrorDialog('ctt_error_dialog_compression_availability_false');
  }
};

prototype._didCreateCompressionDialog = function(e) {
  var query = {};
  var cdc;
  var presets = e.args.data.presets;
  var preset = {};
  var items;
  var show_error_dialog = this._showErrorDialog;

  items = this._contentList.getSelectedItems();
  for (var i = 0; i < items.length; i += 1) {
    if (items[i]['class'] === 'container') {
      items.splice(i, 1);
    }
  }
  if (items.length < 1) {
    this._showErrorDialog('ctt_error_dialog_compression_not_media_selected');
    return;
  }

  for (var i = 0; i < presets.length; i += 1) {
    if (presets[i].selected) {
      preset = presets[i];
    }
  }

  query = {
    title: _('ctt_compression_confirmation_title'),
    description: _('ctt_compression_confirmation_description', preset.name),
    isDelete: false
  };

  query.processing = function() {
    var model = new ContentsTransferApiClient();
    var query = {};
    var error = function() {
      var error_dialog = {
        title: _('ctt_error_dialog_compression_title'),
        description: _('ctt_error_dialog_compression_some_error_description')
      };

      show_error_dialog(error_dialog);
    };

    model.addEventListener('reservationCompressionError', error, this);
    for (var j = 0; j < items.length; j += 1) {
      query.objectId = items[j].id;
      model.addReservationCompression(query);
    }
    this.finish();
  };

  query.finish = dixim.proxy(this._reloadListControllers, this);

  cdc = new ConfirmationDialogController(query);
  cdc.start();
};

prototype._didRequestDeletion = function(e) {
  var vc;
  var items;

  items = this._contentList.getSelectedItems();
  if (!items.length) {
    this._showErrorDialog('ctt_error_dialog_deletion_no_media_selected');
    return;
  }
  if (!this._validateRemovedItems(items)) {
    this._showErrorDialog('ctt_error_dialog_deletion_restricted_media_selected');
    return;
  }

  vc = new DeletionController({targetItems: items});
  vc.addEventListener('close', this._didCloseDeletionDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._validateRemovedItems = function(items) {
  var i;
  var len = items.length;

  for (i = 0; i < len; i += 1) {
    if (!items[i].removable) return false;
    if (!!items[i].restricted) return false;
  }
  return true;
};

prototype._didCloseDeletionDialog = function(e) {
  if (e.args.deleted) {
    this._reloadListControllers();
  }
  this._modal.destroy();
  this._modal = null;
};

prototype._didRequestLocking = function(e) {
  var vc;
  var items;

  items = this._contentList.getSelectedItems();
  if (items.length === 0) {
    this._showErrorDialog('ctt_error_dialog_locking_no_media_selected');
    return;
  }

  vc = new LockingController({targetItems: items});
  vc.addEventListener('close', this._didCloseLockingDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseLockingDialog = function(e) {
  if (e.args.locked) {
    this._reloadListControllers();
  }
  this._modal.destroy();
  this._modal = null;
};

prototype._didRequestUnlocking = function(e) {
  var vc;
  var items;

  items = this._contentList.getSelectedItems();
  if (items.length === 0) {
    this._showErrorDialog('ctt_error_dialog_unlocking_no_media_selected');
    return;
  }

  vc = new UnlockingController({targetItems: items});
  vc.addEventListener('close', this._didCloseUnlockingDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseUnlockingDialog = function(e) {
  if (e.args.unlocked) {
    this._reloadListControllers();
  }
  this._modal.destroy();
  this._modal = null;
};

prototype._didRequestAutoDelSetting = function(e) {
  var vc;
  vc = new AutoDeleteController();
  vc.addEventListener('close', this._didCloseAutoDelSettingDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseAutoDelSettingDialog = function(e) {
  this._modal.destroy();
  this._modal = null;
};

prototype._reloadListControllers = function() {
  this._contentList.reload();
  this._containerList.reload();
};

prototype._hasRestrictedItem = function(items) {
  var len = items.length;
  var i;

  for (i = 0; i < len; i += 1) {
    if (!!items[i].restricted) return true;
  }
  return false;
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._transferStatusMonitor.start();
};

}());
