/**
 * @module ctt
 * @submodule modal-dialog-view
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var $ = dixim.ctt.jQuery;

var ModalDialogView;
var prototype;

/**
 * @class ModalDialogView
 * @extends EventTarget
 * @constructor
 * @param {Object} configs
 */
ModalDialogView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ModalDialogView, EventTarget);
dixim.ctt.ModalDialogView = ModalDialogView;
prototype = ModalDialogView.prototype;

/**
 * The render event is sent when
 * the container has been rendered.
 * 
 * @event render
 */
var RENDER = 'render';

/**
 * Triggered when the page has been closed.
 *
 * @event close
 */
var CLOSE = 'close';

var TEMPLATE_CONTENT_BOX = '<div/>';
var TEMPLATE_BODY_CONTENT = '<div/>';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var cb;

  configs = configs || {};
  cb = typeof configs.showCloseButton === 'undefined' ? true : configs.showCloseButton;

  this._contentBox = configs.contentBox || TEMPLATE_CONTENT_BOX;
  this._bodyContent = configs.bodyContent || TEMPLATE_BODY_CONTENT;
  this._showCloseButton = cb;
  this._buttons = configs.buttons || [];
  this._width = configs.width || 600;
  this._height = configs.height || 340;
  this._minHeight = configs.minHeight;
  this._minWidth = configs.minWidth;
  this._maxHeight = configs.maxHeight;
  this._maxWidth = configs.maxWidth;
  this._title = configs.title || '';
  this._view = null;
  this._overlay = null;
  this._memoryButtonsEvent = {};
};

/**
 * @method _createView
 * @private
 */
prototype._createView = function() {
  var contentBox;
  var bodyContent;
  var dialog;
  var btns;

  contentBox = $(this._contentBox).
                 addClass('ctt-modal-dialog-view-wrapper');
  bodyContent = $(this._bodyContent).
                  addClass('ctt-modal-dialog-view');

  btns = this._convertToDialogButtons(this._buttons);
  dialog = contentBox.append(bodyContent).dialog({
    buttons: btns,
    width: this._width,
    height: this._height,
    minWidth: this._minWidth,
    minHeight: this._minHeight,
    maxWidth: this._maxWidth,
    maxHeight: this._maxHeight,

    // XXX: overlay hack.
    // It seems that the dialog('destroy') also destroys
    // another dialog's overlay that is currently shown.
    // So "modal" is set to false and the overlay is handled
    // manually via this._overlay.
    modal: false,
    // XXX: ...end.
    title: this._title,
    resizable: false,
    autoOpen: false,
    closeOnEscape: false,
    // XXX: Since the create is called before this._view = dialog is
    // evaluated, call did*DialogPre() method.
    create: $.proxy(this._didRenderDialogPre, this),
    open: $.proxy(this._didOpenDialogPre, this)
    // XXX: ...end
  });


  this._view = dialog;
};

prototype._didRenderDialogPre = function(e) {
  this._view = $(e.target);
  this._didRenderDialog(e);
};

prototype._didOpenDialogPre = function(e) {
  this._view = $(e.target);
  this._didOpenDialog(e);
};

/**
 * @method _convertToDialogButtons
 */
prototype._convertToDialogButtons = function(buttons) {
  var rets = [];
  var ret;
  var len = buttons.length;
  var i;
  var button;
  var cls;

  for (i = 0; i < len; i += 1) {
    button = buttons[i];
    ret = dixim.extend({}, button, true);
    if (typeof ret.align !== 'undefined') {
      cls = ret['class'] || '';
      ret['class'] = ['ctt-dialog-view-button ',
                      cls,
                      ' ctt-dialog-view-button-',
                      ret.align].join('');
      delete ret.align;
    }
    rets.push(ret);
  }

  return rets;
};

prototype._didRenderDialog = function(e) {
  var container;
  var cb;

  // XXX: 'e.target' refers bodyContent, not contentBox
  container = $(e.target).parent();

  cb = $('.ui-dialog-titlebar-close', container).
    // XXX: HACK: Since jQuery UI dialog doesn't provide a method
    //      to detect that the right top button is clicked,
    //      detaches the default handler and attaches our own handler instead.
    unbind('click').
    click($.proxy(this._didTitlebarCloseButtonClick, this));
    // XXX: ...end.
  // XXX: ...end.
  if (this._showCloseButton) {
    cb.show();
  } else {
    cb.hide();
  }

  // XXX: overlay hack.
  // $.ui.dialog.overlay should be replace
  // by our own implementation in the future.
  this._overlay = $.ui.dialog.overlay.create(this._view);
  // Since the default overlay keydown handler(attached to the "document")
  // accesses private field of the dialog, disable it to avoid the exception.
  // (Of course, ESC key won't work even if escapeOnEscape is true.)
  $(document).unbind('keydown.dialog-overlay');
  // XXX: ...end.

  this.dispatchEvent(new Event(RENDER));
};

prototype._didOpenDialog = function(e) {
  var overlay = this._overlay;
  var container;

  // XXX: overlay hack.
  container = $(e.target).parent();
  overlay.css('z-index', parseInt(container.css('z-index'), 10) - 1);
  // XXX: ...end.
  $('button').contents().wrap('<a href="#"></a>');
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this.dispatchEvent(new Event(CLOSE));
};

/**
 * @method show
 */
prototype.show = function() {
  if (!!!this._view) {
    this._createView();
  }

  this._view.dialog('open');

  // XXX: overlay hack
  $(this._overlay).show();
  // XXX: ...end.

  this.convertButtons();
};

/**
 * @method hide
 */
prototype.hide = function() {
  this._view.dialog('close');

  // XXX: overlay hack
  $(this._overlay).hide();
  // XXX: ...end.
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  if (!!!this._view) return;

  this._view.dialog('destroy');

  // XXX: overlay hack
  // Since the overlay instance(node) is shared with all dialogs
  // that are created by this class and jQuery.dialog(),
  // it's required to reset 'display' property for
  // if the dialog has been hidden before this destruction.
  this._overlay.css('display', '');
  $.ui.dialog.overlay.destroy(this._overlay);
  // XXX: ...end.

  this._view.remove();
  this._view = null;
};

prototype.getBodyContent = function() {
  return $('.ctt-modal-dialog-view', this._view);
};

prototype.setButtons = function(buttons) {
  this._buttons = buttons || [];
  this._didButtonChange();
};

prototype.convertButtons = function() {
    var default_buttons = [];
    var default_buttons_class = [];
    var default_buttons_id = [];

    $('.ui-dialog-buttonset button', this._view.parent()).each(function() {
        default_buttons.push($(this).html());
        default_buttons_class.push($(this).prop('class'));
        default_buttons_id.push($(this).prop('id'));
    });
    $('.ui-dialog-buttonset button', this._view.parent()).remove();
    for (var i = 0; i < default_buttons.length; i += 1) {
        $('.ui-dialog-buttonset', this._view.parent()).append([
            '<div class="div-btn">',
            default_buttons[i],
            '</div>'
        ].join(''));
        if (default_buttons_class[i] !== '') {
            $('.ui-dialog-buttonset > div:last-child').addClass(default_buttons_class[i]);
        }
        if (default_buttons_id[i] !== '') {
            $('.ui-dialog-buttonset > div:last-child').prop({id: default_buttons_id[i]});
        }
        if (typeof this._buttons[i].click !== 'undefined') {
            $('.ui-dialog-buttonset > div:last-child').click(this._buttons[i].click);
        }
    }
};

prototype._didButtonChange = function() {
  var btns;

  if (!!!this._view) return;
  btns = this._convertToDialogButtons(this._buttons);
  this._view.dialog('option', 'buttons', btns);
  $('button').contents().wrap('<a href="#"></a>');
  this.convertButtons();
};

prototype.showCloseButton = function() {
  $('.ui-dialog-titlebar-close', this._view.parent()).show();
};

prototype.hideCloseButton = function() {
  $('.ui-dialog-titlebar-close', this._view.parent()).hide();
};

prototype.disableButtons = function() {
  var buttonSet;

  buttonSet = $('.ui-dialog-buttonset', this._view.parent());
  $('div.div-btn', buttonSet).attr('disabled', true).
                         addClass('ctt-dialog-view-button-disabled');
};

prototype.enableButtons = function() {
  var buttonSet;

  buttonSet = $('.ui-dialog-buttonset', this._view.parent());
  $('div.div-btn', buttonSet).attr('disabled', false).
                         removeClass('ctt-dialog-view-button-disabled');
};

prototype.enableButton = function(selector) {
  var buttonSet;
  var clickEvent = null;

  buttonSet = $('.ui-dialog-buttonset', this._view.parent());
  $(selector, buttonSet).removeClass('ctt-dialog-view-button-disabled');
  clickEvent = this.getButtonInfo($(selector, buttonSet));
  $(selector, buttonSet).on('click', clickEvent);
  $(selector, buttonSet).find("a").css({cursor: "pointer"}).prop("disabled", false);
};

prototype.disableButton = function(selector) {
  var buttonSet;

  buttonSet = $('.ui-dialog-buttonset', this._view.parent());
  this.setButtonInfo($(selector, buttonSet));
  $(selector, buttonSet).addClass('ctt-dialog-view-button-disabled');
  $(selector, buttonSet).off('click');
  $(selector, buttonSet).find("a").css({cursor: "default"}).prop("disabled", true);
};

prototype.setButtonInfo = function(selector) {
  var selectorText = selector.find('a').text();
  var buttonKey = [selector.closest('.ui-dialog').prop('id'), "&", selectorText].join('');
  var clickEvent = null;

  for (var i = 0; i < this._buttons.length; i += 1) {
    if (this._buttons[i].text === selectorText) {
      clickEvent = this._buttons[i].click;
    }
  }
  this._memoryButtonsEvent[buttonKey] = clickEvent;
}

prototype.getButtonInfo = function(selector) {
  var selectorText = selector.find('a').text();
  var buttonKey = [selector.closest('.ui-dialog').prop('id'), "&", selectorText].join('');
  var clickEvent = null;

  for (var key in this._memoryButtonsEvent) {
    if (buttonKey === key) {
      clickEvent = this._memoryButtonsEvent[key];
    }
  }
  return clickEvent;
};

}());
