/**
 * @module ctt
 * @submodule wizard-page-stack-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;

var WizardPageStackController;
var prototype;

/**
 * Abstract class represents a stack of wizard pages.
 *
 * @class WizardPageStackController
 * @extends EventTarget
 * @constructor
 * @param {Object} configs
 */
WizardPageStackController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(WizardPageStackController, EventTarget);
dixim.ctt.WizardPageStackController = WizardPageStackController;
prototype = WizardPageStackController.prototype;

/**
 * @event done
 */
var DONE = 'done';

/**
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  this._rootController = null;
  this._controllers = [];
  this._registry = {};
};

/**
 * @method pushViewController
 * @param {WizardPageController} vc
 */
prototype.pushViewController = function(vc) {
  return this._pushViewController(vc);
};

/**
 * @method _pushViewController
 * @protected
 * @param {WizardPageController} vc
 */
prototype._pushViewController = function(vc) {
  var list = this._controllers;
  var c;

  if (list.length) {
    c = list[list.length - 1];
    c.stop();
  }

  this._watchViewController(vc);
  list.push(vc);
  vc.start();

  if (!!!this._rootController) {
    this._rootController = vc;
  }
};

prototype._watchViewController = function(vc) {
  vc.addEventListener('done', this._done, this);
  vc.addEventListener('cancel', this._didCancel, this);
  vc.setWizard(this);
};

prototype._unwatchViewController = function(vc) {
  vc.removeEventListener('cancel', this._didCancel, this);
  vc.removeEventListener('done', this._done, this);
  vc.setWizard(null);
};

prototype._done = function(e) {
  this.dispatchEvent(new Event(DONE, {registry: this._registry}));
};

prototype._didCancel = function(e) {
  this.dispatchEvent(new Event(CANCEL));
};

/**
 * @method popViewController
 * @return {WizardPageController}
 */
prototype.popViewController = function() {
  return this._popViewController();
};

/**
 * @method _popViewController
 * @protected
 * @return {WizardPageController}
 */
prototype._popViewController = function() {
  var list = this._controllers;
  var ret;
  var c;

  if (list.length === 1) {
    return;
  }

  ret = list.pop();
  ret.stop();
  this._unwatchViewController(ret);

  c = list[list.length - 1];
  c.start();

  return ret;
};

/**
 * @method replaceTopViewController
 * @param {WizardPageController} vc new top view controller.
 * @return {WizardPageController} past top view controller.
 */
prototype.replaceTopViewController = function(vc) {
  return this._replaceTopViewController(vc);
};

prototype._replaceTopViewController = function(vc) {
  var ret;

  ret = this._controllers.pop();
  ret.stop();
  this._unwatchViewController(ret);
  this._pushViewController(vc);
  return ret;
};

/**
 * @method start
 */
prototype.start = function() {
  if (!this._controllers.length && !!this._rootController) {
    this._pushViewController(this._rootController);
  }
};

/**
 * @method stop
 */
prototype.stop = function() {
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this.destroyAllViewControllers();
  this._registry = null;
};

/**
 * @method destroyAllViewControllers
 */
prototype.destroyAllViewControllers = function() {
  var controllers = this._controllers;
  var i;
  var len = controllers.length;

  for (i = 0; i < len; i += 1) {
    this._unwatchViewController(controllers[i]);
    controllers[i].destroy();
  }
  this._controllers = [];
};

/**
 * Sets/adds an entry to the wizard data registry.
 *
 * @method setRegistryEntry
 * @param {String} key
 * @param {Object} value
 */
prototype.setRegistryEntry = function(key, value) {
  this._registry[key] = value;
};

/**
 * Gets a value of the key specified from the wizard data registry.
 *
 * @method getRegistryEntry
 * @param {String} key;
 * @return {Object}
 */
prototype.getRegistryEntry = function(key) {
  return this._registry[key];
};

}());
